"""This module provides the CLI."""
# cli-module/cli.py
import json
from typing import List, Optional
import typer
from polygon import rest_connect
app = typer.Typer()


@app.command()
def list():
    typer.echo(f"list")
    #call the endpoint to get list of datasets in account
    #rest-connect with the json
    datasetList=rest_connect.dataset_list()
    print(json.dumps(datasetList, indent=3))

@app.command()
def create(name:str = typer.Option("None","--name","--n"),
    classname: Optional[List[str]] = typer.Option("","--classname",),
    clodustoragename: str = typer.Option("None","--clodustoragename","--n"),
) -> None:
    classes=[]
    classnames = classname.strip()
    classnames_list = classnames.split(",")
    if (len(classnames_list) > 0):
        for i in range(len(classnames_list)):
            classes.append(classnames_list[i])
    else:
        classes.append(classname)
    createDataset = rest_connect.createDataset(name, classnames_list, clodustoragename)
    print(createDataset)

#list_datasetname_or_id
@app.command()
def merge(name:str = typer.Option("None","--name","--n"),
    datasetid: str = typer.Option("","--datasetid",),
    datasetname: str = typer.Option("","--datasetname",),
) -> None:
    typer.secho(
        f"""polygon: dataset merge """
        f"""pass in a list of datasetnames or dataset ids""",
        fg=typer.colors.GREEN,
    )
    dataset_id_list=[]
    dataset_name_list=[]
    dataset_id = datasetid.strip()
    dataset_idlist = dataset_id.split(",")
    if (len(dataset_idlist) > 0):
        for i in range(len(dataset_idlist)):
            dataset_id_list.append(dataset_idlist[i])
    else:
        dataset_id_list.append(datasetid)

    dataset_name = datasetname.strip()
    dataset_namelist = dataset_name.split(",")
    if (len(dataset_namelist) > 0):
        for i in range(len(dataset_namelist)):
            dataset_name_list.append(dataset_namelist[i])
    else:
        dataset_name_list.append(datasetname)

    datasetDetails = rest_connect.dataset_merge(dataset_id_list,dataset_name_list, name)
    print(datasetDetails)
    #get the list - the options can be -ids or -names
    # if -ids the list has datasetids -names then the list has datasetnames
    # based on the options get the list and use them to merge the dataset
    #create proper json and call the endpoint to merge dataset


@app.command()
def download(name:str = typer.Option("None","--name","--n"),
    filters: str = typer.Option("","--filters",),
    format: str = typer.Option("","--format",),
    datasetid: str = typer.Option("","--datasetid",),
    datasetname: str = typer.Option("","--datasetname",),
) -> None:
    typer.secho(
        f"""polygon: dataset download """
        f"""pass in a list of datasetnames or dataset ids""",
        fg=typer.colors.GREEN,
    )

    #print(filters)
    filters_list = []
    filters1=filters.strip()
    filter=filters1.split(",")
    if(len(filter)>0):
        for i in range(len(filter)):
            filters_list.append(filter[i])
    else:
        filters_list.append(filters)

    #print(filters_list)

    for id in filters:
        filters_list.append(id)

    if(datasetid =="" and datasetname==""):
        print("Please pass dataset Id or dataset name to download")
    else:
        datasetDownload = rest_connect.dataset_download(datasetid,datasetname, filters_list,format)


@app.command()
def delete(name: str = typer.Option("None","--name"),id: str = typer.Option("None","--id",),)-> None:
    """Add a new to-do with a DESCRIPTION."""
    typer.secho(
        f"""polygon: dataset delete """
        f"""pass datasetname or dataset id""",
        fg=typer.colors.GREEN,
    )
    deletestatus = rest_connect.dataset_delete(name, id)
    print(deletestatus)
    # the options can be -id or -name
    # based on the options take the input and
    #create proper json and call the endpoint to merge dataset

@app.command()
def details(name: str = typer.Option("None","--name"),id: str = typer.Option("None","--id",),)-> None:
    """Add a new to-do with a DESCRIPTION."""
    typer.secho(
        f"""polygon: dataset details """
        f"""pass datasetname or dataset id""",
        fg=typer.colors.GREEN,
    )
    datasetDetails=rest_connect.dataset_details(name,id)
    print(json.dumps(datasetDetails, indent=3))

    # the options can be -id or -name
    # based on the options take the input and
    #create proper json and call the endpoint to merge dataset





if __name__ == "__main__":
    app()



# list_datasetname_or_id: List[str] = typer.Argument(...),
# priority: int = typer.Option(2, "--priority", "-p", min=1, max=3),



import cmd, sys
from turtle import *
from polygon import parse
from polygon import rest_connect
import json
import typer
import simplejson
import traceback
class PolygonShell(cmd.Cmd):
    intro = 'Welcome to the Polygon Query shell.   Type help or ? to list commands.\n'
    prompt = '>'
    file = None

    # ----- basic Polygon commands -----
    def do_select(self, arg):
        'Move the Polygon forward by the specified distance:  FORWARD 10'
        sqlResult=""
        try:
            data = ("**** polygon query **** \n"
                    " ##  Below are the tables that can be queried\n"
                    "        image, dataset, annotation, training_set, version, training_data\n"
                    " ##  Sql query :\n"
                    " ##      Example: \n"
                    "               1. select * from image\n"
                    "               2. select * from dataset limit 5\n"
                    "               3. select objectName from annotation\n"
                    "               4  select name,datasetId from dataset where active=1\n"
                    "               5. select datasetId,name from image where datasetId='<value>' | more\n"
                    " ##  NoSql query : \n"
                    "        Example: \n"
                    "                 db.image.find({'datasetId':'<value>'})\n"
                    "                 db.dataset.find({'name':'<value>'})\n"
                    "                 db.image.count({'datasetId':'<value>'})\n"
                    "                 db.annotation.find({'objectName':'<value>'})")

            if '|' in arg:
                x=arg.split("|")
                args=x[0]
                outptDisplay =True
            else:
                args=arg
                outptDisplay =False
            query="select "+ str(args)
            parsedData=parse.parse_sql(query)
            sqlResult = rest_connect.sqlreslt(parsedData)

            error1 = sqlResult["error"]
            cnt = sqlResult["count"]
            res = sqlResult["results"]
            if (error1 != None):
                print(sqlResult)
                print('\033[92m' +data)
            elif (outptDisplay == True):
                x={
                    "count":cnt,
                    "results":res,
                }
                data = simplejson.dumps(x, indent=4)
                cnt = 0
                for line in data.split('\n'):
                    cnt += 1
                    print(line)
                    input("Press Enter to continue") if cnt % 30 == 0 else None
            else:
                x = {
                    "count": cnt,
                    "results": res,
                }
                print(json.dumps(x, indent=3))


        except Exception:
            #traceback.print_exc()
            print("You have an error in your query syntax")
            print(sqlResult)
            print('\033[92m' +data)

    def do_db(self, arg):
        'Move the Polygon forward by the specified distance:  FORWARD 10'
        nosqlResult=""
        try:
            data = ("**** polygon query **** \n"
                    " ##  Below are the tables that can be queried\n"
                    "        image, dataset, annotation, training_set, version, training_data\n"
                    " ##  Sql query :\n"
                    " ##      Example: \n"
                    "               1. select * from image\n"
                    "               2. select * from dataset limit 5\n"
                    "               3. select objectName from annotation\n"
                    "               4  select name,datasetId from dataset where active=1\n"
                    "               5. select datasetId from image where datasetId='<value>' | more\n"
                    " ##  NoSql query : \n"
                    "        Example: \n"
                    "                 db.image.find({'datasetId':'<value>'})\n"
                    "                 db.dataset.find({'name':'<value>'})\n"
                    "                 db.image.count({'name':'<value>'})\n"
                    "                 db.annotation.find({'objectName':'<value>'})")
            if '|' in arg:
                x = arg.split("|")
                args = x[0]
                outptDisplay = True
            else:
                args = arg
                outptDisplay = False
            pharse="db"+args
            nosqlResult = rest_connect.nosqlreslt(pharse)

            error1=nosqlResult["error"]
            cnt=nosqlResult["count"]
            res=nosqlResult["results"]
            if(error1 !=None):
                print(nosqlResult)
                print('\033[92m' +data)
            elif(outptDisplay == True):
                x = {
                    "count": cnt,
                    "results": res,
                }
                data = simplejson.dumps(x, indent=4)
                cnt = 0
                for line in data.split('\n'):
                    cnt += 1
                    print(line)
                    input("Press Enter to continue") if cnt % 30 == 0 else None
            else:
                x = {
                    "count": cnt,
                    "results": res,
                }
                print(json.dumps(x, indent=3))
        except:
            print("You have an error in your query syntax")
            print(nosqlResult)
            print('\033[92m' +data)


    def do_bye(self, arg):
        'Stop recording, close the  window, and exit:  BYE'
        print(arg)
        print('Thank you for using Polygon')
        self.close()
        bye()
        return True

    def do_clear(self, arg):
        print("\033c")

    def do_help(self, arg):
        data = ("**** polygon query **** \n"                
                    " ##  Below are the tables that can be queried\n"
                    "        image, dataset, annotation, training_set, version, training_data\n"
                    " ##  Sql query :\n"
                    " ##      Example: \n"
                    "               1. select * from image\n"
                    "               2. select * from dataset limit 5\n"
                    "               3. select objectName from annotation\n"
                    "               4  select name,datasetId from dataset where active=1\n"
                    "               5. select datasetId from image where datasetId='<value>' | more\n"
                    " ##  NoSql query : \n"
                    "        Example: \n"
                    "                 db.image.find({'datasetId':'<value>'})\n"
                    "                 db.dataset.find({'name':'<value>'})\n"
                    "                 db.image.count({'name':'<value>'})\n" 
                    "                 db.annotation.find({'objectName':'<value>'})")
        print('\033[92m'+data)

    def pagetext(self,text_lined, num_lines=25):
        for index, line in enumerate(text_lined):
            if index % num_lines == 0 and index:
                input = raw_input("Hit any key to continue press q to quit")
                if input.lower() == 'q':
                    break
            else:
                print(line)

    # ----- record and playback -----
    def do_record(self, arg):
        'Save future commands to filename:  RECORD rose.cmd'
        self.file = open(arg, 'w')
    def do_playback(self, arg):
        'Playback commands from a file:  PLAYBACK rose.cmd'
        self.close()
        with open(arg) as f:
            self.cmdqueue.extend(f.read().splitlines())
    def precmd(self, line):
        # line = line.lower()
        if self.file and 'playback' not in line:
            print(line, file=self.file)
        return line
    def close(self):
        if self.file:
            self.file.close()
            self.file = None

if __name__ == '__main__':
    PolygonShell().cmdloop()


# polygon/__main__.py

from polygon import cli, __app_name__, dataset, container, search

def main():
    cli.app(prog_name=__app_name__)
    # dataset.app(prog_name=__app_name__)
    # container.app(prog_name=__app_name__)

if __name__ == "__main__":
    main()


"""This module provides the CLI."""
# cli-module/cli.py


from typing import List, Optional
import typer
from polygon import rest_connect
app = typer.Typer()


#it is just one command now so taken care in the cli.py

if __name__ == "__main__":
    app()



import sqlparse
import json

def parse_sql(raw):

    # i need table names, select column names and where json
    # Split a string containing two SQL statements:
    statements = sqlparse.split(raw)
    # print(statements)

    # Format the first statement and print it out:
    first = statements[0]
    # print(sqlparse.format(first, reindent=True, keyword_case='upper'))

    # Parsing a SQL statement:
    parsed = sqlparse.parse(first)[0]
    # print(parsed.tokens)
    counter=0
    # tokens_iter = iter(parsed.tokens)
    #TODO IMP NEED TO WRITE A BETTER STATE MACHINE
    STATE = 0
    IN_SELECT = 1
    IN_FROM = 2
    IN_WHERE = 3
    IN_LIMIT = 4

    table = ''
    column_list = []
    where_clause_dict = {}
    limit = -1
    for tok in parsed.tokens:
    # for tok in tokens_iter:
        counter =counter+1
        # print("counter="+str(counter))

        if tok.normalized == 'SELECT':
            STATE = IN_SELECT
        elif tok.normalized == 'FROM':
            STATE = IN_FROM
        elif tok.normalized == 'WHERE' or tok.normalized.startswith('where '):
            STATE = IN_WHERE
        elif tok.normalized == 'LIMIT' or tok.normalized.startswith('where '):
            STATE = IN_LIMIT
        # else:
        #     STATE = 0

        if STATE == IN_SELECT:
            if tok.is_group:
                #print("tok is SELECT group")
                #print(tok)
                for sub_tok in tok.tokens:
                    sub_tok.value = sub_tok.value.strip()
                    #re-check this 2 conditions below. it is very convoluted.
                    #a btter way is to check if they are not space and , and then add, we might not need this complication
                    #check when we have 1, 2,3+ columns
                    if sub_tok.is_group and sub_tok.ttype is None:
                        if not (sub_tok.value=='' or sub_tok.value==',') :
                            column_list.append(sub_tok.value)
                    elif not sub_tok.is_group:
                        if not (sub_tok.value=='' or sub_tok.value==',') :
                            column_list.append(sub_tok.value)
                    # else:
                    #     column_list.append(sub_tok.value)


            #elif tok.normalized == '*':
                #print("tok is SELECT *")
                #print(tok)
                # column_list.append("*")  # if * then no project so pass null to backend


        if STATE == IN_FROM:
            if tok.is_group:
                    #print("tok is FROM group")
                    #print(tok)
                    table = tok.value
                    #we support only 1 table fo r now get that and move to the beginning

        if STATE == IN_LIMIT:
            if (tok.value is not None or len(tok.value)>0)\
                    and tok.value.isnumeric():
                    limit = int(tok.value)

        if STATE == IN_WHERE:
            if tok.is_group:
                for sub_tok in tok.tokens:
                    #print("sub_token=" + str(sub_tok))
                    if sub_tok.normalized =='WHERE':
                        IN_WHERE_SUB_CLAUSE=True
                    if sub_tok.is_group:
                        #handle where clause
                        #print("sub_token||" + str(sub_tok))
                        #print("sub_token||" + str(sub_tok.value.replace('=',':')))
                        #strip quotes out
                        where_clause_dict[sub_tok.left.value] = sub_tok.right.value
                        for sub_sub_tok in sub_tok.tokens:
                            p=0
                            #print("sub_sub_tok=" + str(sub_sub_tok))
                            # pass

                STATE = 0

    where_clause_json_data = json.dumps(where_clause_dict)
    #print("+++++++++")
    output=[]
    output.append(table)
    output.append(column_list)
    output.append(where_clause_json_data)
    output.append(limit)
    #print(output)
    return output




def parse_nonsql(raw):

    #send it as is in the nativeSql portion of the queryrequestbody

    print("+++++++++")
    print(raw)

# [<DML 'select' at 0x7f22c5e15368>, <Whitespace ' ' at 0x7f22c5e153b0>, <Wildcard '*' … ]

raw1 = 'select name, address from foo where d=\'s\' and f=1 and t=\'fgsgdsyufg\' and name="ram"'
raw1 = 'select name, address from foo a, faa b where a.col21=b.col2 and d=\'s\' and f=1 and t=\'fgsgdsyufg\' and name="ram"'
raw1 = 'select name, address from foo a, faa b where a.col21=b.col2 and d=\'s\' and f in (\'abc\',\'def\',\'xyz\')  and t=\'fgsgdsyufg\' and name="ram"'
# raw1 = 'select name from foo where a.col21=b.col2 and d=\'s\' and f in (\'abc\',\'def\',\'xyz\')  and t=\'fgsgdsyufg\' and name="ram"'
# raw1 = 'select * from foo where a.col21=b.col2 and d=\'s\' and f in (\'abc\',\'def\',\'xyz\')  and t=\'fgsgdsyufg\' and name="ram"'
raw1 = 'select * from foo where a.col21=b.col2 and d=\'s\' and f in (\'abc\',\'def\',\'xyz\')  and t=\'fgsgdsyufg\' and name="ram" limit 10'
parse_sql(raw1)

